@@ -1,8 +1,6 @@
-import functools
 import itertools
 import logging
 import os
-import pathlib
 import signal
 import subprocess
 import sys
@@ -11,10 +9,12 @@ import time
 import traceback
 import weakref
 from collections import defaultdict
+from functools import lru_cache, wraps
 from pathlib import Path
 from types import ModuleType
 from zipimport import zipimporter
 
+import django
 from django.apps import apps
 from django.core.signals import request_finished
 from django.dispatch import Signal
@@ -22,11 +22,11 @@ from django.utils.functional import cached_property
 from django.utils.version import get_version_tuple
 
 autoreload_started = Signal()
-file_changed = Signal(providing_args=['file_path', 'kind'])
+file_changed = Signal()
 
-DJANGO_AUTORELOAD_ENV = 'RUN_MAIN'
+DJANGO_AUTORELOAD_ENV = "RUN_MAIN"
 
-logger = logging.getLogger('django.utils.autoreload')
+logger = logging.getLogger("django.utils.autoreload")
 
 # If an error is raised while importing a file, it's not placed in sys.modules.
 # This means that any future modifications aren't caught. Keep a list of these
@@ -46,8 +46,18 @@ except ImportError:
     pywatchman = None
 
 
+def is_django_module(module):
+    """Return True if the given module is nested under Django."""
+    return module.__name__.startswith("django.")
+
+
+def is_django_path(path):
+    """Return True if the given file path is nested under Django."""
+    return Path(django.__file__).parent in Path(path).parents
+
+
 def check_errors(fn):
-    @functools.wraps(fn)
+    @wraps(fn)
     def wrapper(*args, **kwargs):
         global _exception
         try:
@@ -57,7 +67,7 @@ def check_errors(fn):
 
             et, ev, tb = _exception
 
-            if getattr(ev, 'filename', None) is None:
+            if getattr(ev, "filename", None) is None:
                 # get the filename from the last item in the stack
                 filename = traceback.extract_tb(tb)[-1][0]
             else:
@@ -78,19 +88,22 @@ def raise_last_exception():
 
 
 def ensure_echo_on():
-    if termios:
-        fd = sys.stdin
-        if fd.isatty():
-            attr_list = termios.tcgetattr(fd)
-            if not attr_list[3] & termios.ECHO:
-                attr_list[3] |= termios.ECHO
-                if hasattr(signal, 'SIGTTOU'):
-                    old_handler = signal.signal(signal.SIGTTOU, signal.SIG_IGN)
-                else:
-                    old_handler = None
-                termios.tcsetattr(fd, termios.TCSANOW, attr_list)
-                if old_handler is not None:
-                    signal.signal(signal.SIGTTOU, old_handler)
+    """
+    Ensure that echo mode is enabled. Some tools such as PDB disable
+    it which causes usability issues after reload.
+    """
+    if not termios or not sys.stdin.isatty():
+        return
+    attr_list = termios.tcgetattr(sys.stdin)
+    if not attr_list[3] & termios.ECHO:
+        attr_list[3] |= termios.ECHO
+        if hasattr(signal, "SIGTTOU"):
+            old_handler = signal.signal(signal.SIGTTOU, signal.SIG_IGN)
+        else:
+            old_handler = None
+        termios.tcsetattr(sys.stdin, termios.TCSANOW, attr_list)
+        if old_handler is not None:
+            signal.signal(signal.SIGTTOU, old_handler)
 
 
 def iter_all_python_module_files():
@@ -98,12 +111,16 @@ def iter_all_python_module_files():
     # modules based on the module name and pass it to iter_modules_and_files().
     # This ensures cached results are returned in the usual case that modules
     # aren't loaded on the fly.
-    modules_view = sorted(list(sys.modules.items()), key=lambda i: i[0])
-    modules = tuple(m[1] for m in modules_view if not isinstance(m[1], weakref.ProxyTypes))
+    keys = sorted(sys.modules)
+    modules = tuple(
+        m
+        for m in map(sys.modules.__getitem__, keys)
+        if not isinstance(m, weakref.ProxyTypes)
+    )
     return iter_modules_and_files(modules, frozenset(_error_files))
 
 
-@functools.lru_cache(maxsize=1)
+@lru_cache(maxsize=1)
 def iter_modules_and_files(modules, extra_files):
     """Iterate through all modules needed to be watched."""
     sys_file_paths = []
@@ -113,41 +130,47 @@ def iter_modules_and_files(modules, extra_files):
         # cause issues here.
         if not isinstance(module, ModuleType):
             continue
-        if module.__name__ == '__main__':
+        if module.__name__ in ("__main__", "__mp_main__"):
             # __main__ (usually manage.py) doesn't always have a __spec__ set.
             # Handle this by falling back to using __file__, resolved below.
             # See https://docs.python.org/reference/import.html#main-spec
             # __file__ may not exists, e.g. when running ipdb debugger.
-            if hasattr(module, '__file__'):
+            if hasattr(module, "__file__"):
                 sys_file_paths.append(module.__file__)
             continue
-        if getattr(module, '__spec__', None) is None:
+        if getattr(module, "__spec__", None) is None:
             continue
         spec = module.__spec__
         # Modules could be loaded from places without a concrete location. If
         # this is the case, skip them.
         if spec.has_location:
-            origin = spec.loader.archive if isinstance(spec.loader, zipimporter) else spec.origin
+            origin = (
+                spec.loader.archive
+                if isinstance(spec.loader, zipimporter)
+                else spec.origin
+            )
             sys_file_paths.append(origin)
 
     results = set()
     for filename in itertools.chain(sys_file_paths, extra_files):
         if not filename:
             continue
-        path = pathlib.Path(filename)
+        path = Path(filename)
         try:
             if not path.exists():
                 # The module could have been removed, don't fail loudly if this
                 # is the case.
                 continue
-            results.add(path.resolve().absolute())
         except ValueError as e:
             # Network filesystems may return null bytes in file paths.
-            logger.debug('"%s" raised when resolving path: "%s"' % (str(e), path))
+            logger.debug('"%s" raised when resolving path: "%s"', e, path)
+            continue
+        resolved_path = path.resolve().absolute()
+        results.add(resolved_path)
     return frozenset(results)
 
 
-@functools.lru_cache(maxsize=1)
+@lru_cache(maxsize=1)
 def common_roots(paths):
     """
     Return a tuple of common roots that are shared between the given paths.
@@ -187,12 +210,12 @@ def sys_path_directories():
         path = Path(path)
         if not path.exists():
             continue
-        path = path.resolve().absolute()
+        resolved_path = path.resolve().absolute()
         # If the path is a file (like a zip file), watch the parent directory.
-        if path.is_file():
-            yield path.parent
+        if resolved_path.is_file():
+            yield resolved_path.parent
         else:
-            yield path
+            yield resolved_path
 
 
 def get_child_arguments():
@@ -201,30 +224,56 @@ def get_child_arguments():
     executable is reported to not have the .exe extension which can cause bugs
     on reloading.
     """
-    import django.__main__
+    import __main__
 
-    args = [sys.executable] + ['-W%s' % o for o in sys.warnoptions]
-    if sys.argv[0] == django.__main__.__file__:
-        # The server was started with `python -m django runserver`.
-        args += ['-m', 'django']
+    py_script = Path(sys.argv[0])
+
+    args = [sys.executable] + ["-W%s" % o for o in sys.warnoptions]
+    if sys.implementation.name == "cpython":
+        args.extend(
+            f"-X{key}" if value is True else f"-X{key}={value}"
+            for key, value in sys._xoptions.items()
+        )
+    # __spec__ is set when the server was started with the `-m` option,
+    # see https://docs.python.org/3/reference/import.html#main-spec
+    # __spec__ may not exist, e.g. when running in a Conda env.
+    if getattr(__main__, "__spec__", None) is not None:
+        spec = __main__.__spec__
+        if (spec.name == "__main__" or spec.name.endswith(".__main__")) and spec.parent:
+            name = spec.parent
+        else:
+            name = spec.name
+        args += ["-m", name]
         args += sys.argv[1:]
+    elif not py_script.exists():
+        # sys.argv[0] may not exist for several reasons on Windows.
+        # It may exist with a .exe extension or have a -script.py suffix.
+        exe_entrypoint = py_script.with_suffix(".exe")
+        if exe_entrypoint.exists():
+            # Should be executed directly, ignoring sys.executable.
+            return [exe_entrypoint, *sys.argv[1:]]
+        script_entrypoint = py_script.with_name("%s-script.py" % py_script.name)
+        if script_entrypoint.exists():
+            # Should be executed as usual.
+            return [*args, script_entrypoint, *sys.argv[1:]]
+        raise RuntimeError("Script %s does not exist." % py_script)
     else:
         args += sys.argv
     return args
 
 
 def trigger_reload(filename):
-    logger.info('%s changed, reloading.', filename)
+    logger.info("%s changed, reloading.", filename)
     sys.exit(3)
 
 
 def restart_with_reloader():
-    new_environ = {**os.environ, DJANGO_AUTORELOAD_ENV: 'true'}
+    new_environ = {**os.environ, DJANGO_AUTORELOAD_ENV: "true"}
     args = get_child_arguments()
     while True:
-        exit_code = subprocess.call(args, env=new_environ, close_fds=False)
-        if exit_code != 3:
-            return exit_code
+        p = subprocess.run(args, env=new_environ, close_fds=False)
+        if p.returncode != 3:
+            return p.returncode
 
 
 class BaseReloader:
@@ -239,21 +288,14 @@ class BaseReloader:
             path = path.absolute()
         except FileNotFoundError:
             logger.debug(
-                'Unable to watch directory %s as it cannot be resolved.',
+                "Unable to watch directory %s as it cannot be resolved.",
                 path,
                 exc_info=True,
             )
             return
-        logger.debug('Watching dir %s with glob %s.', path, glob)
+        logger.debug("Watching dir %s with glob %s.", path, glob)
         self.directory_globs[path].add(glob)
 
-    def watch_file(self, path):
-        path = Path(path)
-        if not path.is_absolute():
-            raise ValueError('%s must be absolute.' % path)
-        logger.debug('Watching file %s.', path)
-        self.extra_files.add(path)
-
     def watched_files(self, include_globs=True):
         """
         Yield all files that need to be watched, including module files and
@@ -281,13 +323,14 @@ class BaseReloader:
             if app_reg.ready_event.wait(timeout=0.1):
                 return True
         else:
-            logger.debug('Main Django thread has terminated before apps are ready.')
+            logger.debug("Main Django thread has terminated before apps are ready.")
             return False
 
     def run(self, django_main_thread):
-        logger.debug('Waiting for apps ready_event.')
+        logger.debug("Waiting for apps ready_event.")
         self.wait_for_apps_ready(apps, django_main_thread)
         from django.urls import get_resolver
+
         # Prevent a race condition where URL modules aren't loaded when the
         # reloader starts by accessing the urlconf_module property.
         try:
@@ -296,7 +339,7 @@ class BaseReloader:
             # Loading the urlconf can result in errors during development.
             # If this occurs then swallow the error and continue.
             pass
-        logger.debug('Apps ready_event triggered. Sending autoreload_started signal.')
+        logger.debug("Apps ready_event triggered. Sending autoreload_started signal.")
         autoreload_started.send(sender=self)
         self.run_loop()
 
@@ -317,15 +360,15 @@ class BaseReloader:
         testability of the reloader implementations by decoupling the work they
         do from the loop.
         """
-        raise NotImplementedError('subclasses must implement tick().')
+        raise NotImplementedError("subclasses must implement tick().")
 
     @classmethod
     def check_availability(cls):
-        raise NotImplementedError('subclasses must implement check_availability().')
+        raise NotImplementedError("subclasses must implement check_availability().")
 
     def notify_file_changed(self, path):
         results = file_changed.send(sender=self, file_path=path)
-        logger.debug('%s notified as changed. Signal results: %s.', path, results)
+        logger.debug("%s notified as changed. Signal results: %s.", path, results)
         if not any(res[1] for res in results):
             trigger_reload(path)
 
@@ -348,10 +391,15 @@ class StatReloader(BaseReloader):
                 old_time = mtimes.get(filepath)
                 mtimes[filepath] = mtime
                 if old_time is None:
-                    logger.debug('File %s first seen with mtime %s', filepath, mtime)
+                    logger.debug("File %s first seen with mtime %s", filepath, mtime)
                     continue
                 elif mtime > old_time:
-                    logger.debug('File %s previous mtime: %s, current mtime: %s', filepath, old_time, mtime)
+                    logger.debug(
+                        "File %s previous mtime: %s, current mtime: %s",
+                        filepath,
+                        old_time,
+                        mtime,
+                    )
                     self.notify_file_changed(filepath)
 
             time.sleep(self.SLEEP_TIME)
@@ -384,7 +432,7 @@ class WatchmanReloader(BaseReloader):
     def __init__(self):
         self.roots = defaultdict(set)
         self.processed_request = threading.Event()
-        self.client_timeout = int(os.environ.get('DJANGO_WATCHMAN_TIMEOUT', 5))
+        self.client_timeout = int(os.environ.get("DJANGO_WATCHMAN_TIMEOUT", 5))
         super().__init__()
 
     @cached_property
@@ -403,45 +451,63 @@ class WatchmanReloader(BaseReloader):
         # now, watching its parent, if possible, is sufficient.
         if not root.exists():
             if not root.parent.exists():
-                logger.warning('Unable to watch root dir %s as neither it or its parent exist.', root)
+                logger.warning(
+                    "Unable to watch root dir %s as neither it or its parent exist.",
+                    root,
+                )
                 return
             root = root.parent
-        result = self.client.query('watch-project', str(root.absolute()))
-        if 'warning' in result:
-            logger.warning('Watchman warning: %s', result['warning'])
-        logger.debug('Watchman watch-project result: %s', result)
-        return result['watch'], result.get('relative_path')
+        result = self.client.query("watch-project", str(root.absolute()))
+        if "warning" in result:
+            logger.warning("Watchman warning: %s", result["warning"])
+        logger.debug("Watchman watch-project result: %s", result)
+        return result["watch"], result.get("relative_path")
 
-    @functools.lru_cache()
+    @lru_cache
     def _get_clock(self, root):
-        return self.client.query('clock', root)['clock']
+        return self.client.query("clock", root)["clock"]
 
     def _subscribe(self, directory, name, expression):
         root, rel_path = self._watch_root(directory)
+        # Only receive notifications of files changing, filtering out other types
+        # like special files: https://facebook.github.io/watchman/docs/type
+        only_files_expression = [
+            "allof",
+            ["anyof", ["type", "f"], ["type", "l"]],
+            expression,
+        ]
         query = {
-            'expression': expression,
-            'fields': ['name'],
-            'since': self._get_clock(root),
-            'dedup_results': True,
+            "expression": only_files_expression,
+            "fields": ["name"],
+            "since": self._get_clock(root),
+            "dedup_results": True,
         }
         if rel_path:
-            query['relative_root'] = rel_path
-        logger.debug('Issuing watchman subscription %s, for root %s. Query: %s', name, root, query)
-        self.client.query('subscribe', root, name, query)
+            query["relative_root"] = rel_path
+        logger.debug(
+            "Issuing watchman subscription %s, for root %s. Query: %s",
+            name,
+            root,
+            query,
+        )
+        self.client.query("subscribe", root, name, query)
 
     def _subscribe_dir(self, directory, filenames):
         if not directory.exists():
             if not directory.parent.exists():
-                logger.warning('Unable to watch directory %s as neither it or its parent exist.', directory)
+                logger.warning(
+                    "Unable to watch directory %s as neither it or its parent exist.",
+                    directory,
+                )
                 return
-            prefix = 'files-parent-%s' % directory.name
-            filenames = ['%s/%s' % (directory.name, filename) for filename in filenames]
+            prefix = "files-parent-%s" % directory.name
+            filenames = ["%s/%s" % (directory.name, filename) for filename in filenames]
             directory = directory.parent
-            expression = ['name', filenames, 'wholename']
+            expression = ["name", filenames, "wholename"]
         else:
-            prefix = 'files'
-            expression = ['name', filenames]
-        self._subscribe(directory, '%s:%s' % (prefix, directory), expression)
+            prefix = "files"
+            expression = ["name", filenames]
+        self._subscribe(directory, "%s:%s" % (prefix, directory), expression)
 
     def _watch_glob(self, directory, patterns):
         """
@@ -452,19 +518,22 @@ class WatchmanReloader(BaseReloader):
         overwrite the named subscription, so it must include all possible glob
         expressions.
         """
-        prefix = 'glob'
+        prefix = "glob"
         if not directory.exists():
             if not directory.parent.exists():
-                logger.warning('Unable to watch directory %s as neither it or its parent exist.', directory)
+                logger.warning(
+                    "Unable to watch directory %s as neither it or its parent exist.",
+                    directory,
+                )
                 return
-            prefix = 'glob-parent-%s' % directory.name
-            patterns = ['%s/%s' % (directory.name, pattern) for pattern in patterns]
+            prefix = "glob-parent-%s" % directory.name
+            patterns = ["%s/%s" % (directory.name, pattern) for pattern in patterns]
             directory = directory.parent
 
-        expression = ['anyof']
+        expression = ["anyof"]
         for pattern in patterns:
-            expression.append(['match', pattern, 'wholename'])
-        self._subscribe(directory, '%s:%s' % (prefix, directory), expression)
+            expression.append(["match", pattern, "wholename"])
+        self._subscribe(directory, "%s:%s" % (prefix, directory), expression)
 
     def watched_roots(self, watched_files):
         extra_directories = self.directory_globs.keys()
@@ -475,8 +544,8 @@ class WatchmanReloader(BaseReloader):
     def _update_watches(self):
         watched_files = list(self.watched_files(include_globs=False))
         found_roots = common_roots(self.watched_roots(watched_files))
-        logger.debug('Watching %s files', len(watched_files))
-        logger.debug('Found common roots: %s', found_roots)
+        logger.debug("Watching %s files", len(watched_files))
+        logger.debug("Found common roots: %s", found_roots)
         # Setup initial roots for performance, shortest roots first.
         for root in sorted(found_roots):
             self._watch_root(root)
@@ -486,7 +555,9 @@ class WatchmanReloader(BaseReloader):
         sorted_files = sorted(watched_files, key=lambda p: p.parent)
         for directory, group in itertools.groupby(sorted_files, key=lambda p: p.parent):
             # These paths need to be relative to the parent directory.
-            self._subscribe_dir(directory, [str(p.relative_to(directory)) for p in group])
+            self._subscribe_dir(
+                directory, [str(p.relative_to(directory)) for p in group]
+            )
 
     def update_watches(self):
         try:
@@ -500,19 +571,19 @@ class WatchmanReloader(BaseReloader):
         subscription = self.client.getSubscription(sub)
         if not subscription:
             return
-        logger.debug('Watchman subscription %s has results.', sub)
+        logger.debug("Watchman subscription %s has results.", sub)
         for result in subscription:
             # When using watch-project, it's not simple to get the relative
             # directory without storing some specific state. Store the full
             # path to the directory in the subscription name, prefixed by its
             # type (glob, files).
-            root_directory = Path(result['subscription'].split(':', 1)[1])
-            logger.debug('Found root directory %s', root_directory)
-            for file in result.get('files', []):
+            root_directory = Path(result["subscription"].split(":", 1)[1])
+            logger.debug("Found root directory %s", root_directory)
+            for file in result.get("files", []):
                 self.notify_file_changed(root_directory / file)
 
     def request_processed(self, **kwargs):
-        logger.debug('Request processed. Setting update_watches event.')
+        logger.debug("Request processed. Setting update_watches event.")
         self.processed_request.set()
 
     def tick(self):
@@ -524,12 +595,17 @@ class WatchmanReloader(BaseReloader):
                 self.processed_request.clear()
             try:
                 self.client.receive()
+            except pywatchman.SocketTimeout:
+                pass
             except pywatchman.WatchmanError as ex:
+                logger.debug("Watchman error: %s, checking server status.", ex)
                 self.check_server_status(ex)
             else:
                 for sub in list(self.client.subs.keys()):
                     self._check_subscription(sub)
             yield
+            # Protect against busy loops.
+            time.sleep(0.1)
 
     def stop(self):
         self.client.close()
@@ -538,7 +614,7 @@ class WatchmanReloader(BaseReloader):
     def check_server_status(self, inner_ex=None):
         """Return True if the server is available."""
         try:
-            self.client.query('version')
+            self.client.query("version")
         except Exception:
             raise WatchmanUnavailable(str(inner_ex)) from inner_ex
         return True
@@ -546,19 +622,19 @@ class WatchmanReloader(BaseReloader):
     @classmethod
     def check_availability(cls):
         if not pywatchman:
-            raise WatchmanUnavailable('pywatchman not installed.')
+            raise WatchmanUnavailable("pywatchman not installed.")
         client = pywatchman.client(timeout=0.1)
         try:
             result = client.capabilityCheck()
         except Exception:
             # The service is down?
-            raise WatchmanUnavailable('Cannot connect to the watchman service.')
-        version = get_version_tuple(result['version'])
+            raise WatchmanUnavailable("Cannot connect to the watchman service.")
+        version = get_version_tuple(result["version"])
         # Watchman 4.9 includes multiple improvements to watching project
         # directories as well as case insensitive filesystems.
-        logger.debug('Watchman version %s', version)
+        logger.debug("Watchman version %s", version)
         if version < (4, 9):
-            raise WatchmanUnavailable('Watchman 4.9 or later is required.')
+            raise WatchmanUnavailable("Watchman 4.9 or later is required.")
 
 
 def get_reloader():
@@ -574,27 +650,24 @@ def start_django(reloader, main_func, *args, **kwargs):
     ensure_echo_on()
 
     main_func = check_errors(main_func)
-    django_main_thread = threading.Thread(target=main_func, args=args, kwargs=kwargs, name='django-main-thread')
-    django_main_thread.setDaemon(True)
+    django_main_thread = threading.Thread(
+        target=main_func, args=args, kwargs=kwargs, name="django-main-thread"
+    )
+    django_main_thread.daemon = True
     django_main_thread.start()
 
     while not reloader.should_stop:
-        try:
-            reloader.run(django_main_thread)
-        except WatchmanUnavailable as ex:
-            # It's possible that the watchman service shuts down or otherwise
-            # becomes unavailable. In that case, use the StatReloader.
-            reloader = StatReloader()
-            logger.error('Error connecting to Watchman: %s', ex)
-            logger.info('Watching for file changes with %s', reloader.__class__.__name__)
+        reloader.run(django_main_thread)
 
 
 def run_with_reloader(main_func, *args, **kwargs):
     signal.signal(signal.SIGTERM, lambda *args: sys.exit(0))
     try:
-        if os.environ.get(DJANGO_AUTORELOAD_ENV) == 'true':
+        if os.environ.get(DJANGO_AUTORELOAD_ENV) == "true":
             reloader = get_reloader()
-            logger.info('Watching for file changes with %s', reloader.__class__.__name__)
+            logger.info(
+                "Watching for file changes with %s", reloader.__class__.__name__
+            )
             start_django(reloader, main_func, *args, **kwargs)
         else:
             exit_code = restart_with_reloader()
