@@ -613,6 +613,39 @@ class TestUtilsHashPass(SimpleTestCase):
             check_password("wrong_password", encoded)
             self.assertEqual(hasher.harden_runtime.call_count, 1)
 
+    def test_check_password_calls_make_password_to_fake_runtime(self):
+        hasher = get_hasher("default")
+        cases = [
+            (None, None, None),  # no plain text password provided
+            ("foo", make_password(password=None), None),  # unusable encoded
+            ("letmein", make_password(password="letmein"), ValueError),  # valid encoded
+        ]
+        for password, encoded, hasher_side_effect in cases:
+            with self.subTest(encoded=encoded):
+                with mock.patch(
+                    "django.contrib.auth.hashers.identify_hasher",
+                    side_effect=hasher_side_effect,
+                ) as mock_identify_hasher:
+                    with mock.patch(
+                        "django.contrib.auth.hashers.make_password"
+                    ) as mock_make_password:
+                        with mock.patch(
+                            "django.contrib.auth.hashers.get_random_string",
+                            side_effect=lambda size: "x" * size,
+                        ):
+                            with mock.patch.object(hasher, "verify"):
+                                # make_password() is called to standardize timing.
+                                check_password(password, encoded)
+                                self.assertEqual(hasher.verify.call_count, 0)
+                                self.assertEqual(
+                                    mock_identify_hasher.mock_calls,
+                                    [mock.call(encoded)],
+                                )
+                                self.assertEqual(
+                                    mock_make_password.mock_calls,
+                                    [mock.call("x" * UNUSABLE_PASSWORD_SUFFIX_LENGTH)],
+                                )
+
     def test_encode_invalid_salt(self):
         hasher_classes = [
             MD5PasswordHasher,
