@@ -11,8 +11,8 @@ class SimpleDatabaseClientTests(SimpleTestCase):
 
     def test_settings_to_cmd_args_env(self):
         msg = (
-            'subclasses of BaseDatabaseClient must provide a '
-            'settings_to_cmd_args_env() method or override a runshell().'
+            "subclasses of BaseDatabaseClient must provide a "
+            "settings_to_cmd_args_env() method or override a runshell()."
         )
         with self.assertRaisesMessage(NotImplementedError, msg):
             self.client.settings_to_cmd_args_env(None, None)
@@ -20,10 +20,10 @@ class SimpleDatabaseClientTests(SimpleTestCase):
     def test_runshell_use_environ(self):
         for env in [None, {}]:
             with self.subTest(env=env):
-                with mock.patch('subprocess.run') as run:
+                with mock.patch("subprocess.run") as run:
                     with mock.patch.object(
                         BaseDatabaseClient,
-                        'settings_to_cmd_args_env',
+                        "settings_to_cmd_args_env",
                         return_value=([], env),
                     ):
                         self.client.runshell(None)
