@@ -5,16 +5,15 @@ from django.test.utils import modify_settings, override_settings
 
 
 @override_settings(
-    ROOT_URLCONF='flatpages_tests.urls',
+    ROOT_URLCONF="flatpages_tests.urls",
     SITE_ID=1,
 )
 @modify_settings(
     INSTALLED_APPS={
-        'append': ['django.contrib.sitemaps', 'django.contrib.flatpages'],
+        "append": ["django.contrib.sitemaps", "django.contrib.flatpages"],
     },
 )
 class FlatpagesSitemapTests(TestCase):
-
     @classmethod
     def setUpClass(cls):
         super().setUpClass()
@@ -24,15 +23,20 @@ class FlatpagesSitemapTests(TestCase):
 
     @classmethod
     def setUpTestData(cls):
-        Site = apps.get_model('sites.Site')
+        Site = apps.get_model("sites.Site")
         current_site = Site.objects.get_current()
         current_site.flatpage_set.create(url="/foo/", title="foo")
-        current_site.flatpage_set.create(url="/private-foo/", title="private foo", registration_required=True)
+        current_site.flatpage_set.create(
+            url="/private-foo/", title="private foo", registration_required=True
+        )
 
     def test_flatpage_sitemap(self):
-        response = self.client.get('/flatpages/sitemap.xml')
-        self.assertIn(b'<url><loc>http://example.com/flatpage_root/foo/</loc></url>', response.getvalue())
+        response = self.client.get("/flatpages/sitemap.xml")
+        self.assertIn(
+            b"<url><loc>http://example.com/flatpage_root/foo/</loc></url>",
+            response.getvalue(),
+        )
         self.assertNotIn(
-            b'<url><loc>http://example.com/flatpage_root/private-foo/</loc></url>',
+            b"<url><loc>http://example.com/flatpage_root/private-foo/</loc></url>",
             response.getvalue(),
         )
